"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault").default;
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createSettingsCrud = void 0;
var _mergeWith = _interopRequireDefault(require("lodash/mergeWith"));
var _error = _interopRequireDefault(require("@webiny/error"));
var _get = _interopRequireDefault(require("lodash/get"));
var _dataloader = _interopRequireDefault(require("dataloader"));
var _pubsub = require("@webiny/pubsub");
var _validation = require("./settings/validation");
var _utils = require("@webiny/utils");
/**
 * Possible types of settings.
 * If a lot of types should be added maybe we can do it via the plugin.
 */
var SETTINGS_TYPE = /*#__PURE__*/function (SETTINGS_TYPE) {
  SETTINGS_TYPE["DEFAULT"] = "default";
  return SETTINGS_TYPE;
}(SETTINGS_TYPE || {});
const createSettingsCrud = params => {
  const {
    storageOperations,
    getLocaleCode,
    getTenantId
  } = params;
  const settingsDataLoader = new _dataloader.default(async keys => {
    const promises = keys.map(key => {
      const params = {
        where: key
      };
      return storageOperations.settings.get(params);
    });
    return await Promise.all(promises);
  }, {
    cacheKeyFn: key => {
      return [`T#${key.tenant}`, `L#${key.locale}`].join("#");
    }
  });
  const onSettingsBeforeUpdate = (0, _pubsub.createTopic)("pageBuilder.onSettingsBeforeUpdate");
  const onSettingsAfterUpdate = (0, _pubsub.createTopic)("pageBuilder.onSettingsAfterUpdate");
  return {
    onSettingsBeforeUpdate,
    onSettingsAfterUpdate,
    async getCurrentSettings() {
      // With this line commented, we made this endpoint public.
      // We did this because of the public website pages which need to access the settings.
      // It's possible we'll create another GraphQL field, made for this exact purpose.
      // auth !== false && (await checkBasePermissions(context));

      const current = await this.getSettings();
      const defaults = await this.getDefaultSettings();
      return (0, _mergeWith.default)({}, defaults, current, (prev, next) => {
        // No need to use falsy value if we have it set in the default settings.
        if (prev && !next) {
          return prev;
        }
      });
    },
    async getSettings() {
      // With this line commented, we made this endpoint public.
      // We did this because of the public website pages which need to access the settings.
      // It's possible we'll create another GraphQL field, made for this exact purpose.
      // auth !== false && (await checkBasePermissions(context));

      const key = {
        tenant: getTenantId(),
        locale: getLocaleCode()
      };
      try {
        return await settingsDataLoader.load(key);
      } catch (ex) {
        throw new _error.default(ex.message || "Could not get settings by given parameters.", ex.code || "GET_SETTINGS_ERROR", key);
      }
    },
    async getDefaultSettings() {
      return await storageOperations.settings.getDefaults();
    },
    async updateSettings(input) {
      const params = {
        tenant: getTenantId(),
        locale: getLocaleCode(),
        type: SETTINGS_TYPE.DEFAULT
      };
      let original = await this.getSettings();
      if (!original) {
        const data = {
          input: input,
          settings: {
            name: "",
            prerendering: {
              app: {
                url: ""
              },
              storage: {
                name: ""
              },
              meta: {}
            },
            websiteUrl: "",
            pages: {
              home: null,
              notFound: null
            },
            ...params
          }
        };
        try {
          original = await storageOperations.settings.create(data);
          /**
           * Clear the cache of the data loader.
           */
          settingsDataLoader.clearAll();
        } catch (ex) {
          throw new _error.default(ex.message || "Could not create settings record.", ex.code || "CREATE_SETTINGS_ERROR", data);
        }
      }
      const validation = await (0, _validation.createSettingsCreateValidation)().safeParseAsync(input);
      if (!validation.success) {
        throw (0, _utils.createZodError)(validation.error);
      }
      const data = (0, _utils.removeUndefinedValues)(validation.data);
      const settings = {
        ...original,
        ...data,
        ...params
      };

      // Before continuing, let's check for differences that matter.

      // 1. Check differences in `pages` property (`home`, `notFound`). If there are
      // differences, check if the pages can be set as the new `specialType` page, and then,
      // after save, make sure to trigger events, on which other plugins can do their tasks.
      const specialTypes = ["home", "notFound"];
      const changedPages = [];
      for (let i = 0; i < specialTypes.length; i++) {
        const specialType = specialTypes[i];
        const previousPage = (0, _get.default)(original, `pages.${specialType}`);
        const nextPage = (0, _get.default)(settings, `pages.${specialType}`);
        if (previousPage !== nextPage) {
          // Only throw if previously we had a page (previousPage), and now all of a sudden
          // we don't (!nextPage). Allows updating settings without sending these.
          if (previousPage && !nextPage) {
            throw new _error.default(`Cannot unset "${specialType}" page. Please provide a new page if you want to unset current one.`, "CANNOT_UNSET_SPECIAL_PAGE");
          }

          // Only load if the next page (nextPage) has been sent, which is always a
          // must if previously a page was defined (previousPage).
          if (nextPage) {
            const page = await this.getPublishedPageById({
              id: nextPage
            });
            changedPages.push([specialType, previousPage, nextPage, page]);
          }
        }
      }
      const meta = {
        diff: {
          pages: changedPages
        }
      };
      try {
        await onSettingsBeforeUpdate.publish({
          original,
          settings,
          meta
        });
        const result = await storageOperations.settings.update({
          input: input,
          original,
          settings
        });
        await onSettingsAfterUpdate.publish({
          original,
          settings,
          meta
        });
        /**
         * Clear the cache of the data loader.
         */
        settingsDataLoader.clearAll();
        return result;
      } catch (ex) {
        throw new _error.default(ex.message || "Could not update existing settings record.", ex.code || "UPDATE_SETTINGS_ERROR", {
          original,
          settings
        });
      }
    }
  };
};
exports.createSettingsCrud = createSettingsCrud;

//# sourceMappingURL=settings.crud.js.map